module NeuralNetwork (
    input  logic                  clk,
    input  logic                  reset,
    input  logic signed [31:0]    in1, 
    input  logic signed [31:0]    in2, 
    input  logic signed [31:0]    in3, 
    input  logic signed [31:0]    in4,
    output logic signed [31:0]    out1, 
    output logic signed [31:0]    out2, 
    output logic signed [31:0]    out3
);

    //--------------------------------------------------------------------------
    //  Функция ReLU на 32-битном входе (возвращаем также 32 бита)
    //--------------------------------------------------------------------------
    function logic signed [31:0] ReLU(input logic signed [31:0] x);
        begin
            if (x > 0)
                ReLU = x;
            else
                ReLU = 32'sd0;
        end
    endfunction

    //--------------------------------------------------------------------------
    //  Первый слой (W1: 4×8, B1: 8)
    //
    //  Исходные данные для весов заданы как 8 строк по 4 числа:
    //    [
    //      [  0,    0,    0,    0  ],
    //      [  0,  -36,  -66,    0  ],
    //      [  0,    0,  -28,  -32  ],
    //      [ -56,   0,    0,    0  ],
    //      [ -36,   0,   44,  -65  ],
    //      [  0,   83,    0,  -100 ],
    //      [  38,  -37,  127,   118 ],
    //      [  78,   0,   -93,    0  ]
    //    ]
    //  Для W1 требуется транспонировать: строки → столбцы.
    //--------------------------------------------------------------------------
    logic signed [7:0] W1 [3:0][7:0] = '{
        // веса от in1 для нейронов 0..7 (1-й элемент каждой строки)
        '{8'sd0, 8'sd0, 8'sd0, -8'sd56, -8'sd36, 8'sd0, 8'sd38, 8'sd78},
        // веса от in2 для нейронов 0..7 (2-й элемент каждой строки)
        '{8'sd0, -8'sd36, 8'sd0, 8'sd0, 8'sd0, 8'sd83, -8'sd37, 8'sd0},
        // веса от in3 для нейронов 0..7 (3-й элемент каждой строки)
        '{8'sd0, -8'sd66, -8'sd28, 8'sd0, 8'sd44, 8'sd0, 8'sd127, -8'sd93},
        // веса от in4 для нейронов 0..7 (4-й элемент каждой строки)
        '{8'sd0, 8'sd0, -8'sd32, 8'sd0, -8'sd65, -8'sd100, 8'sd118, 8'sd0}
    };

    // Смещения первого слоя (B1: 8)
    // Исходные данные: [0, 0, 0, 0, 0, 497, -529, 1849]
    logic signed [7:0] B1 [7:0] = '{
         8'sd0,
         8'sd0,
         8'sd0,
         8'sd0,
         8'sd0,
         8'sd497,
        -8'sd529,
         8'sd1849
    };

    //--------------------------------------------------------------------------
    //  Второй слой (W2: 8×8, B2: 8)
    //
    //  Исходные данные для весов второго слоя (8 строк по 8 чисел):
    //    [
    //      [ 49,   0,    0,   44,  -45,    0,  -40,   0 ],
    //      [  0,   0,  -34,    0,    0,    0,  -33,   0 ],
    //      [  0,  46,  -28,    0,   35,  -45,  -34,  21 ],
    //      [  0,  47,   32,    0,  -24,  -62,   63,   0 ],
    //      [  0,  25,    0,  -32,  -35,    0,    0,   0 ],
    //      [  0,   0,   22,  -25,  -46,    0,    0, 127 ],
    //      [ 22,   0,   42,    0,   35,    0,    0,   0 ],
    //      [  0, -24,  -30,   27,    0,   46,  -37,   0 ]
    //    ]
    //--------------------------------------------------------------------------
    logic signed [7:0] W2 [7:0][7:0] = '{
        '{8'sd49,  8'sd0,   8'sd0,   8'sd44,  -8'sd45, 8'sd0,  -8'sd40, 8'sd0},
        '{8'sd0,   8'sd0,  -8'sd34,  8'sd0,   8'sd0,   8'sd0,  -8'sd33, 8'sd0},
        '{8'sd0,   8'sd46,  -8'sd28,  8'sd0,   8'sd35,  -8'sd45, -8'sd34, 8'sd21},
        '{8'sd0,   8'sd47,   8'sd32,  8'sd0,  -8'sd24, -8'sd62, 8'sd63,  8'sd0},
        '{8'sd0,   8'sd25,   8'sd0,  -8'sd32, -8'sd35, 8'sd0,   8'sd0,   8'sd0},
        '{8'sd0,   8'sd0,   8'sd22, -8'sd25, -8'sd46, 8'sd0,   8'sd0,   8'sd127},
        '{8'sd22,  8'sd0,   8'sd42,  8'sd0,   8'sd35, 8'sd0,   8'sd0,   8'sd0},
        '{8'sd0,  -8'sd24, -8'sd30, 8'sd27,  8'sd0,   8'sd46, -8'sd37, 8'sd0}
    };

    // Смещения второго слоя (B2: 8)
    // Исходные данные: [0, -99, 0, -321, 0, 568, 0, -42]
    logic signed [7:0] B2 [7:0] = '{
         8'sd0,
        -8'sd99,
         8'sd0,
        -8'sd321,
         8'sd0,
         8'sd568,
         8'sd0,
        -8'sd42
    };

    //--------------------------------------------------------------------------
    //  Выходной слой (W3: 8×3, B3: 3)
    //
    //  Исходные данные для весов выходного слоя заданы как 3 строки по 8 чисел:
    //    [
    //      [  14,  -8,   38, -127,   -9,  110,  -43,  -30 ],
    //      [  30,  50,   -2,  -14,   52,   39,  -44,  -70 ],
    //      [ -39,  23,   19,   33,   53, -114,  -35,   52 ]
    //    ]
    //  Транспонируем в массив W3[8][3].
    //--------------------------------------------------------------------------
    logic signed [7:0] W3 [7:0][2:0] = '{
        // Вес от hidden2[0] ко всем выходам
        '{8'sd14,   8'sd30,   -8'sd39},
        // hidden2[1]
        '{-8'sd8,   8'sd50,    8'sd23},
        // hidden2[2]
        '{8'sd38,   -8'sd2,   8'sd19},
        // hidden2[3]
        '{-8'sd127, -8'sd14,  8'sd33},
        // hidden2[4]
        '{-8'sd9,   8'sd52,   8'sd53},
        // hidden2[5]
        '{8'sd110,  8'sd39,   -8'sd114},
        // hidden2[6]
        '{-8'sd43,  -8'sd44,  -8'sd35},
        // hidden2[7]
        '{-8'sd30,  -8'sd70,   8'sd52}
    };

    // Смещения выходного слоя (B3: 3)
    // Исходные данные: [251, 335, -614]
    logic signed [7:0] B3 [2:0] = '{
         8'sd251,
         8'sd335,
        -8'sd614
    };

    //--------------------------------------------------------------------------
    //  Промежуточные сигналы скрытых слоёв (32 бита, чтобы вместить сумму)
    //--------------------------------------------------------------------------
    logic signed [31:0] hidden1 [7:0];
    logic signed [31:0] hidden2 [7:0];

    //--------------------------------------------------------------------------
    //  Основной процесс: вычисление выхода нейросети
    //--------------------------------------------------------------------------
    always_ff @(posedge clk or posedge reset) begin
        if (reset) begin
            out1 <= 32'sd0;
            out2 <= 32'sd0;
            out3 <= 32'sd0;
        end 
        else begin
            //------------------------------------------------------------------
            //  Первый скрытый слой (8 нейронов)
            //------------------------------------------------------------------
            for (int i = 0; i < 8; i++) begin
                hidden1[i] <= ReLU(
                    ($signed(W1[0][i]) * in1) +
                    ($signed(W1[1][i]) * in2) +
                    ($signed(W1[2][i]) * in3) +
                    ($signed(W1[3][i]) * in4) +
                    $signed(B1[i])
                );
            end

            //------------------------------------------------------------------
            //  Второй скрытый слой (8 нейронов)
            //------------------------------------------------------------------
            for (int i = 0; i < 8; i++) begin
                hidden2[i] <= ReLU(
                    hidden1[0]*$signed(W2[0][i]) +
                    hidden1[1]*$signed(W2[1][i]) +
                    hidden1[2]*$signed(W2[2][i]) +
                    hidden1[3]*$signed(W2[3][i]) +
                    hidden1[4]*$signed(W2[4][i]) +
                    hidden1[5]*$signed(W2[5][i]) +
                    hidden1[6]*$signed(W2[6][i]) +
                    hidden1[7]*$signed(W2[7][i]) +
                    $signed(B2[i])
                );
            end

            //------------------------------------------------------------------
            //  Выходной слой (3 выхода)
            //------------------------------------------------------------------
            out1 <= hidden2[0]*$signed(W3[0][0]) +
                    hidden2[1]*$signed(W3[1][0]) +
                    hidden2[2]*$signed(W3[2][0]) +
                    hidden2[3]*$signed(W3[3][0]) +
                    hidden2[4]*$signed(W3[4][0]) +
                    hidden2[5]*$signed(W3[5][0]) +
                    hidden2[6]*$signed(W3[6][0]) +
                    hidden2[7]*$signed(W3[7][0]) +
                    $signed(B3[0]);

            out2 <= hidden2[0]*$signed(W3[0][1]) +
                    hidden2[1]*$signed(W3[1][1]) +
                    hidden2[2]*$signed(W3[2][1]) +
                    hidden2[3]*$signed(W3[3][1]) +
                    hidden2[4]*$signed(W3[4][1]) +
                    hidden2[5]*$signed(W3[5][1]) +
                    hidden2[6]*$signed(W3[6][1]) +
                    hidden2[7]*$signed(W3[7][1]) +
                    $signed(B3[1]);

            out3 <= hidden2[0]*$signed(W3[0][2]) +
                    hidden2[1]*$signed(W3[1][2]) +
                    hidden2[2]*$signed(W3[2][2]) +
                    hidden2[3]*$signed(W3[3][2]) +
                    hidden2[4]*$signed(W3[4][2]) +
                    hidden2[5]*$signed(W3[5][2]) +
                    hidden2[6]*$signed(W3[6][2]) +
                    hidden2[7]*$signed(W3[7][2]) +
                    $signed(B3[2]);
        end
    end
endmodule